
#include "inc.h"

#include "minix.h"
#include "iwram_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "script.h"
#include "screen.h"
#include "main.h"

//#include "header\font\BGFont.h"


//Mode3
//Spr J[\
//---------------------------------------------------------------------------
//screen.c
extern ST_SCREEN Screen;

//---------------------------------------------------------------------------
u8  State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:  StateInit();  break;
			case STATE_RESET: StateReset(); break;
			case STATE_EXEC:  StateExec();  break;
			case STATE_DRAW:  StateDraw();  break;
			case STATE_WAIT:  StateWait();  break;
			case STATE_MENU:  StateMenu();  break;

			default:
				DebugMessage("Err: Main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitKey();
	InitIRQ();
	InitFont();
	InitSprite();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER0;

	REG_TM0D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM0CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	ST_FONT_SRC sing;		//Sp
	ST_FONT_SRC half;		//p

	sing.pDat   = (u8*)&mplus_j10rBitmap;
	sing.pSheet = (u16*)&mplus_jfnt_txt;
	sing.imgCx  = MPLUS_J10R_IMG_CX;
	sing.cnt    = MPLUS_J10R_FONT_CNT;
	sing.cx     = MPLUS_J10R_FONT_CX;
	sing.cy     = MPLUS_J10R_FONT_CY;

	half.pDat   = (u8*)&mplus_s10rBitmap;
	half.pSheet = (u16*)&mplus_sfnt_txt;
	half.imgCx  = MPLUS_S10R_IMG_CX;
	half.cnt    = MPLUS_S10R_FONT_CNT;
	half.cx     = MPLUS_S10R_FONT_CX;
	half.cy     = MPLUS_S10R_FONT_CY;

	Mode3SetFont(&sing, &half);
	Mode3SetFontDrawBuffer((u16*)&Screen.buf);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();

	SpriteSetPalData16((u16*)&sprPal);
	SpriteSetData((u16*)&sprTiles, sprTilesLen);
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	if( GBFSInit() == FALSE )
	{
		ErrorMessage("GBFS file not found.", NULL);
	}

	ScriptInit();
	ScreenInit();
	CursorInit();

	State = STATE_EXEC;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateExec()
{
	ScriptExec();

	State = ( ScriptIsEnd() == TRUE ? STATE_RESET : STATE_DRAW );
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateDraw()
{
	ScreenCalc();		//`Ȍs܂
	ScreenDraw();

	State = STATE_WAIT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateWait()
{
	ScreenWait();

	State = STATE_MENU;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateMenu()
{
	ScreenMenu();

	State = STATE_EXEC;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ErrorMessage(char* msg1, char* msg2)
{
	Mode3SetFontDrawBuffer(VideoBuffer);

	if(msg1 != NULL) Mode3DrawFontStr(0, 0, (u8*)msg1, COLOR_WHITE);
	if(msg2 != NULL) Mode3DrawFontStr(0,11, (u8*)msg2, COLOR_WHITE);

	for(;;){}
}
