
#include "inc.h"

#include "minix.h"
#include "iwram_arm.h"
#include "div_arm.h"
#include "bios_arm.h"
#include "gbfs.h"

#include "main.h"
#include "lib.h"
#include "script.h"
#include "screen.h"

//---------------------------------------------------------------------------
//script.c
extern ST_SCRIPT Script;
//lib.c
extern ST_FONT Font;

//---------------------------------------------------------------------------
ST_SCREEN Screen EWRAM_DATA;
ST_CURSOR Cursor EWRAM_DATA;

//---------------------------------------------------------------------------
EWRAM_CODE void ScreenInit()
{
	_Memset((u8*)&Screen, 0x00, sizeof(ST_SCREEN));
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenInitMsg()
{
	_Memset((u8*)&Screen.msg, 0x00, SCREEN_MSG_LINE * SCREEN_MSG_LEN);
	Screen.msgLine = 0;
}
//---------------------------------------------------------------------------
//SCRIPT̃tO߂܂i`ȎOj
EWRAM_CODE void ScreenCalc()
{
	ST_SCRIPT* p = &Script;
	ST_SCREEN* s = &Screen;

	switch(p->flagCmd)
	{
	case SCRIPT_FLAG_CLEAR:
		ScreenCalcClear(p->flagSub);
		s->flagUpdate = TRUE;
		break;

	case SCRIPT_FLAG_MESSAGE:
		ScreenCalcMsg(p->str);
		s->flagMsg    = TRUE;
		s->flagUpdate = TRUE;
		s->flagWait   = TRUE;
		break;

	case SCRIPT_FLAG_UPDATE:
		s->flagUpdate = TRUE;
		break;

	case SCRIPT_FLAG_LOAD:
		ScreenCalcLoad(p->str, p->flagSub);
		break;

	case SCRIPT_FLAG_MENU_INIT:
		ScreenCalcMenuClear();
		break;

	case SCRIPT_FLAG_MENU_ITEM:
		ScreenCalcMenuSetItem(p->str, p->flagSub);
		break;

	case SCRIPT_FLAG_MENU:
		ScreenCalcMenu(p->flagSub);
		s->flagMsg    = TRUE;
		s->flagUpdate = TRUE;
		s->flagMenu   = TRUE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCalcMsg(u8* msg)
{
	ScreenInitMsg();

	ST_SCREEN* p = &Screen;
	ST_FONT*   f = &Font;

	u16 x   = 0;	//ϐɏވʒu
	u16 y   = 0;
	u16 ix  = 0;	//XN[ɏލWʒu

//	TRACEOUT("ScreenCalcMsg srcAdr = %x  dstAdr = %x\n", msg, &p->msg[0][0]);

	for(;;)
	{
		if( (*msg == '\0') || (y >= SCREEN_MSG_LINE) )
		{
			p->msg[y][x] = '\0';
			p->msgLine   = y;
			return;
		}

		if(*msg == '\n')
		{
			msg++;
			p->msg[y][x] = '\0';
			x  = 0;
			ix = 0;
			y++;

			continue;
		}

		//QoCg`FbN܂
		if(IsSJIS(*msg) == TRUE)
		{
			if(x+2+1 >= SCREEN_MSG_LEN || ix + f->sing.cx >= SCREEN_CX)		//+1'\0'
			{
				p->msg[y][x] = '\0';
				x  = 0;
				ix = 0;
				y++;
			}
			else
			{
				p->msg[y][x++] = *msg++;
				p->msg[y][x++] = *msg++;
				ix += f->sing.cx;
			}
		}
		else
		{
			if(x+1+1 >= SCREEN_MSG_LEN || ix + f->half.cx >= SCREEN_CX)
			{
				p->msg[y][x] = '\0';
				x  = 0;
				ix = 0;
				y++;
			}
			else
			{
				p->msg[y][x++] = *msg++;
				ix += f->half.cx;
			}
		}
	} // for(;;)
}
//---------------------------------------------------------------------------
//t@C݂邩̃`FbN͂܂Ał̓XN[obt@ɓǂݍ݂Ȃł
EWRAM_CODE void ScreenCalcLoad(u8* str, u16 pos)
{
	ST_SCREEN* p = &Screen;
	u8* pCg      = GBFSGetFilePointer((char*)str);

	if(pCg == NULL)
	{
		ErrorMessage("cg file not found.", (char*)str);
	}

//TRACEOUT("pos = %d\n", pos);

	switch(pos)
	{
	case POSITION_BACKONLY:
//		p->flagBack   = FALSE;
//		p->flagLeft   = FALSE;
//		p->flagRight  = FALSE;
		//not break;

	case POSITION_BACK:
		p->flagBack   = TRUE;
		p->fpBack     = pCg;
		break;

	case POSITION_CENTER:
		p->flagCenter = TRUE;
		p->fpCenter   = pCg;
		break;

	case POSITION_LEFT:
		p->flagLeft   = TRUE;
		p->fpLeft     = pCg;
		break;

	case POSITION_RIGHT:
		p->flagRight  = TRUE;
		p->fpRight    = pCg;
		break;

//	case POSITION_OVERLAP: TODO
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCalcClear(u16 pos)
{
	ST_SCREEN* p = &Screen;

	switch(pos)
	{
	case POSITION_BACKONLY:
		p->flagBack   = FALSE;	//TODO ĂH
		p->flagLeft   = FALSE;
		p->flagRight  = FALSE;
		//not break;

	case POSITION_BACK:
		p->flagBack   = FALSE;
		break;

	case POSITION_CENTER:
		p->flagCenter = FALSE;
		break;

	case POSITION_LEFT:
		p->flagLeft   = FALSE;
		break;

	case POSITION_RIGHT:
		p->flagRight  = FALSE;
		break;

//	case POSITION_OVERLAP: TODO
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenDraw()
{
	ST_SCREEN* p = &Screen;

	if(p->flagUpdate == FALSE)
	{
		return;
	}
	p->flagUpdate = FALSE;


	ScreenBufCls();

	if(p->flagBack   == TRUE) ScreenBufCopyBack();
	if(p->flagRight  == TRUE) ScreenBufCopyRight();
	if(p->flagLeft   == TRUE) ScreenBufCopyLeft();
	if(p->flagCenter == TRUE) ScreenBufCopyCenter();
	if(p->flagMsg    == TRUE) ScreenBufDrawMsg();

	WaitForVsync();
	ScreenVRAMCopyBuf();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenVRAMCopyBuf()
{
	REG_DM3SAD = (u32)&Screen.buf;
	REG_DM3DAD = (u32)VideoBuffer;
	REG_DM3CNT = (240 * 160 / 2)
	             | ( (DMA_SIZE_32 | DMA_TRANSFER_ON | DMA_TIMING_NOW | DMA_REPEAT_OFF |
	                  DMA_SAD_INC | DMA_DAD_INC     | DMA_INTR_OFF) << 16 );
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCls()
{
	REG_DM2SAD = (u32)&Screen.dummyU32;
	REG_DM2DAD = (u32)&Screen.buf;
	REG_DM2CNT = (240 * 160 / 2)
	             | ( (DMA_SIZE_32 | DMA_TRANSFER_ON | DMA_TIMING_NOW | DMA_REPEAT_OFF |
	                  DMA_SAD_FIX | DMA_DAD_INC     | DMA_INTR_OFF) << 16 );
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufDrawMsg()
{
	ST_SCREEN* p = &Screen;
	ST_FONT*   f = &Font;

	//bZ[WEBhEÂƂ܂
	ScreenBufDrawMsgWindow();

	u16 i;
	for(i=0; i<p->msgLine; i++)
	{
		Mode3DrawFontStr(SCREEN_MSG_SX, SCREEN_MSG_SY+(f->sing.cy * i), (u8*)p->msg[i], COLOR_WHITE);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyBack()
{
	ST_SCREEN* p = &Screen;

	LZ77UnCompWram(p->fpBack, p->buf);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyRight()
{
	ST_SCREEN* p = &Screen;

	LZ77UnCompWram(p->fpRight, p->chr);
	ScreenBufFillCopy(120, 0, 120, 160, (u16*)p->chr);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyLeft()
{
	ST_SCREEN* p = &Screen;

	LZ77UnCompWram(p->fpLeft, p->chr);
	ScreenBufFillCopy(0, 0, 120, 160, (u16*)p->chr);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufCopyCenter()
{
	ST_SCREEN* p = &Screen;

	LZ77UnCompWram(p->fpCenter, p->chr);
	ScreenBufFillCopy(60, 0, 120, 160, (u16*)p->chr);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufFillCopy(u16 sx, u16 sy, u16 cx, u16 cy, u16* img)
{
	u16* scr = (u16*)Screen.buf + (sy * SCREEN_CX) + sx;
	u16  x,y;

	for(y=0; y<cy; y++)
	{
		for(x=0; x<cx; x++)
		{
			if(img[y*cx + x] == COLOR_GREEN)
			{
				continue;
			}

			scr[y*SCREEN_CX + x] = img[y*cx + x];
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBufDrawMsgWindow()
{
	u16* sp = (u16*)&Screen.buf[SCREEN_MSG_SY][SCREEN_MSG_SX];
	u16* ep = (u16*)&Screen.buf[SCREEN_CY][SCREEN_CX];
	u16  col, r, g, b;

	while(sp < ep)
	{
		col = *sp;

		r = ((col >>  0) & 31);
		g = ((col >>  5) & 31);
		b = ((col >> 10) & 31);

		r = (r >> 2) + 6;  //TODO ̏ꂵ̂iOOG
		g = (g >> 2) + 6;
		b = (b >> 2) + 6;

		*sp++ = RGB(r, g, b);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenWait()
{
	ST_SCREEN* p = &Screen;

	if(p->flagWait == FALSE)
	{
		return;
	}
	p->flagWait = FALSE;


	KeyWait(KEY_A | KEY_R);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCalcMenuClear()
{
	ST_SCREEN* p = &Screen;

	p->menuCnt = 0;
}
//---------------------------------------------------------------------------
//1IɂA\1sƂɂĂ܂
EWRAM_CODE void ScreenCalcMenuSetItem(u8* str, u16 ans)
{
	ST_SCREEN*      s = &Screen;
	ST_SCREEN_MENU* p = &Screen.menu[s->menuCnt];
	u16 len = _Strlen(str);

	if(s->menuCnt+1 >= MENU_MAX_ITEM)
	{
		return;
	}

	if(len+1 <= SCREEN_MSG_LEN)
	{
		str[SCREEN_MSG_LEN-1] = '\0';
		_Memcpy(p->str, str, SCREEN_MSG_LEN);
	}
	else
	{
		_Memcpy(p->str, str, len+1);
	}

	p->ans = ans;
	p->len = len;
	s->menuCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCalcMenu(u16 adr)
{
	ST_SCREEN* p = &Screen;

	u16 i;
	for(i=0; i<p->menuCnt; i++)
	{
		//TODO \̃̕`FbNĂȂ
		//TODO Ip̃EBhEׂ
		_Strncpy((u8*)&p->msg[i][0], (u8*)"@@", 4);
		_Strncpy((u8*)&p->msg[i][4], p->menu[i].str, SCREEN_MSG_LEN-4);

		TRACEOUT("Screen MenuItem[%d] = %s\n", i, p->msg[i]);
	}

	p->msgLine = p->menuCnt;
	p->menuAdr = adr;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenMenu()
{
	ST_CURSOR* p = &Cursor;
	ST_SCREEN* s = &Screen;
	u16 key;

	if(s->flagMenu == FALSE)
	{
		return;
	}
	s->flagMenu = FALSE;


	CursorMoveIn();

	for(;;)
	{
		WaitForVsync();
		key = KeyGet();

		if(key == KEY_NONE) continue;

		if(key & KEY_A)
		{
			ScriptCmdSetValue(s->menuAdr, s->menu[p->selCnt].ans);
			CursorMoveOut();
			return;
		}
		else if(key & KEY_UP)
		{
			if(p->selCnt == 0) continue;

			p->selCnt--;
			CursorDraw();
		}
		else if(key & KEY_DOWN)
		{
			if(p->selCnt+1 >= s->menuCnt) continue;

			p->selCnt++;
			CursorDraw();
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorInit()
{
	ST_CURSOR* p = &Cursor;

	p->x       = SCREEN_CX;
	p->y       = SCREEN_CY;
	p->cx      = 16;
	p->cy      = 11;
	p->sprNo   = 0;
	p->sprTile = CURSOR_TILENO;
	p->sprSize = SP_SIZE_16;
	p->sprForm = SP_SQUARE;
	p->selCnt  = 0;

	SpriteSetChr (p->sprNo, p->sprTile);
	SpriteSetSize(p->sprNo, p->sprSize, p->sprForm, SP_COLOR_16);
	SpriteMove   (p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorDraw()
{
	ST_CURSOR* p = &Cursor;

	SpriteMove(p->sprNo, p->x, p->y + p->cy * p->selCnt);
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorMoveIn()
{
	ST_CURSOR* p = &Cursor;

	p->x      = SCREEN_MSG_SX;
	p->y      = SCREEN_MSG_SY;
	p->selCnt = 0;

	CursorDraw();
}
//---------------------------------------------------------------------------
EWRAM_CODE void CursorMoveOut()
{
	ST_CURSOR* p = &Cursor;

	p->x = SCREEN_CX;
	p->y = SCREEN_CY;

	CursorDraw();
}
