
#include "inc.h"

#include "minix.h"
#include "iwram_arm.h"
#include "div_arm.h"
#include "gbfs.h"

#include "lib.h"
#include "main.h"
#include "script.h"

//---------------------------------------------------------------------------
ST_SCRIPT Script EWRAM_DATA;

//---------------------------------------------------------------------------
EWRAM_CODE void ScriptInit()
{
	ScriptCmdLoad((u8*)"main");
	ScriptCmdClear();
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScriptIsScr(u8* pFile)
{
	return _Strncmp(pFile, (u8*)"[SCRIPT]", 8) == 0 ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptCmdLoad(u8* str)
{
	ST_SCRIPT* s = &Script;
	u8* pScr;

	_Strcpy(s->scrName, str);
	_Strcat(s->scrName, (u8*)".scr");
	pScr = GBFSGetFilePointer((char*)s->scrName);

	if(pScr == NULL || ScriptIsScr(pScr) == FALSE)
	{
		ErrorMessage("scr not found.", (char*)str);
	}

	ScriptLoad(pScr);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptLoad(u8* pScr)
{
	ST_SCRIPT_HEADER* h = (ST_SCRIPT_HEADER*)pScr;
	ST_SCRIPT*        s = &Script;

	s->len     = h->len;
	s->pos     = (u8*)(h + 1);
	s->cnt     = 0;
	s->str     = NULL;
	s->flagCmd = 0x0000;
	s->flagSub = 0x0000;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptExec()
{
	ST_SCRIPT* s = &Script;
	ST_CMD* c;
	s32 l, r;

	for(;;)
	{
		c = ScriptGetCmd();

		switch(c->common.type)
		{
		case CMD_VALUE_SET:
			ScriptCmdSetValue(c->set.valueAdr, c->set.setValue);
			break;

		case CMD_CALC_VALUE:
			ScriptCmdCalcValue(c->calc.valueAdr, c->calc.addValue);
			break;

		case CMD_TEXT:
			s->flagCmd = SCRIPT_FLAG_MESSAGE;
			s->str     = ScriptCmdGetString(c->text.msgLen);
			return;

		case CMD_CLEAR_TEXT:
			s->flagCmd = SCRIPT_FLAG_CLEAR_TEXT;
			return;

		case CMD_GOTO:
			ScriptCmdGoto(c->cmdGoto.gotoLabel);
			break;

		case CMD_IF_TRUE:
			l = ScriptCmdGetValue(c->cmdIf.value1, c->cmdIf.flag & 1);
			r = ScriptCmdGetValue(c->cmdIf.value2, c->cmdIf.flag & 2);

			if(l == r)
			{
				 ScriptCmdGoto(c->cmdIf.gotoLabel);
			}
			break;

		case CMD_IF_FALSE:
			l = ScriptCmdGetValue(c->cmdIf.value1, c->cmdIf.flag & 1);
			r = ScriptCmdGetValue(c->cmdIf.value2, c->cmdIf.flag & 2);

			if(l != r)
			{
				 ScriptCmdGoto(c->cmdIf.gotoLabel);
			}
			break;

		case CMD_IF_BIGGER:
			l = ScriptCmdGetValue(c->cmdIf.value1, c->cmdIf.flag & 1);
			r = ScriptCmdGetValue(c->cmdIf.value2, c->cmdIf.flag & 2);

			if(l > r)
			{
				 ScriptCmdGoto(c->cmdIf.gotoLabel);
			}
			break;
		
		case CMD_IF_SMALLER:
			l = ScriptCmdGetValue(c->cmdIf.value1, c->cmdIf.flag & 1);
			r = ScriptCmdGetValue(c->cmdIf.value2, c->cmdIf.flag & 2);

			if(l < r)
			{
				 ScriptCmdGoto(c->cmdIf.gotoLabel);
			}
			break;

		case CMD_IF_BIGGER_EQU:
			l = ScriptCmdGetValue(c->cmdIf.value1, c->cmdIf.flag & 1);
			r = ScriptCmdGetValue(c->cmdIf.value2, c->cmdIf.flag & 2);

			if(l >= r)
			{
				 ScriptCmdGoto(c->cmdIf.gotoLabel);
			}
			break;

		case CMD_IF_SMALLER_EQU:
			l = ScriptCmdGetValue(c->cmdIf.value1, c->cmdIf.flag & 1);
			r = ScriptCmdGetValue(c->cmdIf.value2, c->cmdIf.flag & 2);

			if(l <= r)
			{
				 ScriptCmdGoto(c->cmdIf.gotoLabel);
			}
			break;

		case CMD_MENU_INIT:
			s->flagCmd = SCRIPT_FLAG_MENU_INIT;
			return;

		case CMD_MENU_ITEM:
			s->flagCmd = SCRIPT_FLAG_MENU_ITEM;
			s->flagSub = c->menuItem.number;
			s->str     = ScriptCmdGetString(c->menuItem.labelLen);
			return;

		case CMD_MENU:
			s->flagCmd = SCRIPT_FLAG_MENU;
			s->flagSub = c->menu.valueAdr;
			return;

		case CMD_EXEC:
			s->str     = ScriptCmdGetString(c->menuItem.labelLen);
			ScriptCmdLoad(Script.str);
			break;

		case CMD_LOAD:
			s->flagCmd = SCRIPT_FLAG_LOAD;
			s->flagSub = c->load.flag;
			s->str     = ScriptCmdGetString(c->load.pathLen);
			return;

		case CMD_UPDATE:
			s->flagCmd = SCRIPT_FLAG_UPDATE;
			return;

		case CMD_CLEAR:
			s->flagCmd = SCRIPT_FLAG_CLEAR;
			s->flagSub = c->clear.pos;
			return;

		case CMD_MODE:
			//TODO
			break;

		case CMD_SYS_EXIT:
			s->flagCmd = SCRIPT_FLAG_END;
			break;

		case CMD_SYS_CLEAR:
			ScriptCmdClear();
			break;

		case CMD_END:
			s->flagCmd = SCRIPT_FLAG_END;
			return;

		default:
			ErrorMessage("token error.", NULL);
			break;

		}// switch type

	}// for(;;)
}
//---------------------------------------------------------------------------
EWRAM_CODE ST_CMD* ScriptGetCmd()
{
	ST_SCRIPT* p = &Script;

	ST_CMD* c  = (ST_CMD*)(p->pos + p->cnt);
	p->cnt    += c->common.size;

	TRACEOUT("---------------------------\n");
	TRACEOUT("ScriptGetCmd Adr = %x Next = %x\n", c, c + p->cnt);
	TRACEOUT("ScriptGetCmd Token = %02x\n", c->common.type);

	return c;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptCmdSetValue(u16 adr, s32 value)
{
	Script.valueTbl[ adr ] = value;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptCmdCalcValue(u16 adr, s32 value)
{
	Script.valueTbl[ adr ] += value;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptCmdGoto(u32 gotoCnt)
{
	Script.cnt = gotoCnt;
}
//---------------------------------------------------------------------------
EWRAM_CODE s32 ScriptCmdGetValue(s32 adr, u8 flag)
{
	return flag | 0x00 ? adr : Script.valueTbl[ adr ];
}
//---------------------------------------------------------------------------
EWRAM_CODE u8* ScriptCmdGetString(u16 msgLen)
{
	ST_SCRIPT* p = &Script;
	u8* ret;

	if(msgLen == 0)
	{
		return NULL;
	}

	ret = (u8*)(p->pos + p->cnt);
	p->cnt += msgLen;

	TRACEOUT("ScriptCmdGetString Adr = %x\n", ret);
	TRACEOUT("ScriptCmdGetString Str = %s\n", ret);

	return ret;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScriptCmdClear()
{
	_Memset((u8*)&Script.valueTbl, 0x00, SCRIPT_MAX_VALUE_CNT);
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScriptIsEnd()
{
	return Script.flagCmd == SCRIPT_FLAG_END ? TRUE : FALSE;
}
